/* epkowa_ip.h -- for scanners that don't speak EPSON ESC/I natively
   Copyright (C) 2005  Olaf Meeuwissen
*/

#ifndef epkowa_ip_h_included
#define epkowa_ip_h_included

#include <sys/types.h>
#include <sane/sane.h>
#include <ltdl.h>

#include "epkowa.h"
#include "epkowa_ip_api.h"

/* Insulate the interpreter_type declaration against changes.
 */
typedef Epson_Device  device_type;


/*! An object-like type for ESC/I interpreters.
 */
struct interpreter_type
{
  /* public members */

  SANE_Status (*dtor) (device_type *device);

  int (*open)  (device_type *device);
  int (*close) (device_type *device);

  ssize_t (*recv) (device_type *device, void *buffer, size_t size,
		   SANE_Status *status);
  ssize_t (*send) (device_type *device, void *buffer, size_t size,
		   SANE_Status *status);

  /* FIXME: very questionable public API for an _interpreter_ as this
     seems to do some kind of image processing -> suggest to refactor
     this out into a separate object */

  SANE_Status (*free) (device_type *device);

  SANE_Status (*ftor0) (device_type *device, SANE_Parameters *params,
			SANE_Byte *ptr, SANE_Byte *end);
  SANE_Status (*ftor1) (device_type *device, SANE_Parameters *params,
			int depth, int left, int resolution);


  /* private members */

  device_type *_device;
  void	      *_module;

  int _tick_count;

  double    *_table;
  SANE_Byte *_buffer;

  /* hooks for all API declared in epkowa_ip_api.h */

  bool (*_init)  (int fd, io_callback *read, io_callback *write);
  void (*_fini)  (void);
  int  (*_read)  (void *buf, size_t size);
  int  (*_write) (void *buf, size_t size);
  void (*_power) (void);
  int  (*_s_0)   (unsigned int offset,	   unsigned int width,
		  unsigned int resolution, unsigned int opt_resolution,
		  double *table);
  void (*_s_1)   (uint8_t *in_buf, uint8_t *out_buf,
		  unsigned int width, bool color, double *table);
};

/*! Checks for connected interpreter-supported devices.
 */
void find_interpreted_devices (SANE_Status (*attach) (const char *dev));

/*! An abstract, factory-like ctor for interpreters.
 */
SANE_Status create_interpreter (device_type *device,
				unsigned int usb_product_id);

#endif /* !epkowa_ip_h_included */
